package ru.volgait2015.strepetov.portknocking;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Random;
import java.util.logging.Logger;

/**
 * Represents an implementation of port-knocking server
 */
public class Server {

    private static final int PORT_LOW_BOUND = 45000;
    private static final int PORT_HIGH_BOUND = 46000;

    private static final int KNOCK_LOW_BOUND = 10;
    private static final int KNOCK_HIGH_BOUND = 50;

    private static final Logger log = Logger.getLogger(Server.class.getSimpleName());

    public static void main(String[] args) throws IOException {

        int newPort = generateRandomNumberInBound(PORT_LOW_BOUND, PORT_HIGH_BOUND);
        sendNewPort(PORT_LOW_BOUND, newPort);
        int port = newPort;

        int knockCount = generateRandomNumberInBound(KNOCK_LOW_BOUND, KNOCK_HIGH_BOUND);
        log.info("Knock count for this connection will be:"+knockCount);

        for (int i = 0; i < knockCount; i++) {
            newPort = generateRandomNumberInBound(PORT_LOW_BOUND, PORT_HIGH_BOUND);
            sendNewPort(port, newPort);
            port = newPort;
        }

        sendNewPort(port, PORT_HIGH_BOUND);

        sendHelloMessage();

    }

    /**
     * Connects to the client on oldPort and sent to client message with newPort
     * @param oldPort port for ServerSocket
     * @param newPort port, which should be sent to the client
     * @throws IOException
     */
    private static void sendNewPort(int oldPort, int newPort) throws IOException {
        ServerSocket serverSocket = null;

        try {
            serverSocket = new ServerSocket(oldPort);
        } catch (IOException e) {
            log.severe("Could not listen on Port: " + oldPort);
            System.exit(1);
        }
        Socket clientSocket = null;
        log.info("Server is waiting for connection");

        try {
            clientSocket = serverSocket.accept();
        } catch (IOException e) {
            log.severe("Accept failed.");
            System.exit(1);
        }

        log.info("Connection was successful with client:"+clientSocket.getInetAddress());

        PrintWriter out = new PrintWriter(clientSocket.getOutputStream(),
                true);
        BufferedReader in = new BufferedReader(
                new InputStreamReader(clientSocket.getInputStream()));

        out.println(newPort);

        out.close();
        in.close();
        clientSocket.close();
        serverSocket.close();
    }

    /**
     * Connects to the client on port PORT_HIGH_BOUND and sent to the client message with hello and current time
     * @throws IOException
     */
    private static void sendHelloMessage() throws IOException {
        ServerSocket serverSocket = null;

        try {
            serverSocket = new ServerSocket(PORT_HIGH_BOUND);
        } catch (IOException e) {
            log.severe("Could not listen on Port: " + PORT_LOW_BOUND);
            System.exit(1);
        }
        Socket clientSocket = null;
        log.info("Server is waiting for connection");

        try {
            clientSocket = serverSocket.accept();
        } catch (IOException e) {
            log.severe("Accept failed.");
            System.exit(1);
        }

        log.info("Connection was successful with client:"+clientSocket.getInetAddress());

        PrintWriter out = new PrintWriter(clientSocket.getOutputStream(),
                true);
        BufferedReader in = new BufferedReader(
                new InputStreamReader(clientSocket.getInputStream()));

        out.println("Hello "+getCurrentTime());

        out.close();
        in.close();
        clientSocket.close();
        serverSocket.close();
    }

    /**
     * Returns pseudo-random number between lowBound and highBound
     * @param lowBound must be positive
     * @param highBound must be postive
     * @return generated pseudo-random number
     */
    private static int generateRandomNumberInBound(int lowBound, int highBound) {
        Random r = new Random();
        return r.nextInt(highBound - lowBound) + lowBound;
    }

    /**
     * Returns current time in special format
     * @return formatted current time
     */
    private static String getCurrentTime() {
        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("YYYY:MM:DD HH:mm:SS");

        return simpleDateFormat.format(calendar.getTime());
    }
}
